@@ -83,6 +83,11 @@ impl<I: Input, S: Storage<Item = I>> Network<I, S> {
         self.compact(node_filter);
     }
 
+    /// Finds node by its coordinate.
+    pub fn find(&self, coordinate: &Coordinate) -> Option<&NodeLink<I, S>> {
+        self.nodes.get(coordinate)
+    }
+
     /// Returns node coordinates in arbitrary order.
     pub fn get_coordinates(&'_ self) -> impl Iterator<Item = Coordinate> + '_ {
         self.nodes.keys().cloned()
@@ -93,6 +98,11 @@ impl<I: Input, S: Storage<Item = I>> Network<I, S> {
         self.nodes.values()
     }
 
+    /// Iterates over coordinates and their nodes.
+    pub fn iter(&self) -> impl Iterator<Item = (&Coordinate, &NodeLink<I, S>)> {
+        self.nodes.iter()
+    }
+
     /// Returns a total amount of nodes.
     pub fn size(&self) -> usize {
         self.nodes.len()
@@ -251,36 +261,49 @@ impl<I: Input, S: Storage<Item = I>> Network<I, S> {
     }
 
     fn compact(&mut self, node_filter: &(dyn Fn(&NodeLink<I, S>) -> bool)) {
-        let mut remove = vec![];
+        let original = self.nodes.len();
+        let mut removed = vec![];
         let mut remove_node = |coordinate: &Coordinate, node: &mut NodeLink<I, S>| {
-            let topology = &mut node.write().unwrap().topology;
-            topology.left.iter_mut().for_each(|link| link.write().unwrap().topology.right = None);
-            topology.right.iter_mut().for_each(|link| link.write().unwrap().topology.left = None);
-            topology.up.iter_mut().for_each(|link| link.write().unwrap().topology.down = None);
-            topology.down.iter_mut().for_each(|link| link.write().unwrap().topology.up = None);
-
-            remove.push(coordinate.clone());
+            if (original - removed.len()) > 4 {
+                Self::disconnect_node(node);
+                removed.push(coordinate.clone());
+                Ok(())
+            } else {
+                Err(())
+            }
         };
 
         // remove user defined nodes
-        self.nodes
+        let _ = self
+            .nodes
             .iter_mut()
-            .filter(|(_, node)| node_filter.deref()(node))
-            .for_each(|(coordinate, node)| remove_node(coordinate, node));
+            .filter(|(_, node)| !node_filter.deref()(node))
+            .try_for_each(|(coordinate, node)| remove_node(coordinate, node));
+
         // remove empty nodes which are not at boundary
-        self.nodes
+        let _ = self
+            .nodes
             .iter_mut()
             .filter(|(_, node)| {
                 let node = node.read().unwrap();
                 node.storage.size() == 0 && node.topology.is_boundary()
             })
-            .for_each(|(coordinate, node)| remove_node(coordinate, node));
+            .try_for_each(|(coordinate, node)| remove_node(coordinate, node));
 
-        remove.iter().for_each(|coordinate| {
+        removed.iter().for_each(|coordinate| {
             self.nodes.remove(coordinate);
         });
     }
 
+    /// Disconnects node from network.
+    fn disconnect_node(node: &mut NodeLink<I, S>) {
+        let topology = &mut node.write().unwrap().topology;
+        topology.left.iter_mut().for_each(|link| link.write().unwrap().topology.right = None);
+        topology.right.iter_mut().for_each(|link| link.write().unwrap().topology.left = None);
+        topology.up.iter_mut().for_each(|link| link.write().unwrap().topology.down = None);
+        topology.down.iter_mut().for_each(|link| link.write().unwrap().topology.up = None);
+    }
+
     /// Creates nodes for initial topology.
     fn create_initial_nodes(
         roots: [I; 4],
@@ -320,3 +343,11 @@ impl<I: Input, S: Storage<Item = I>> Network<I, S> {
             .collect()
     }
 }
+
+impl<I: Input, S: Storage<Item = I>> Drop for Network<I, S> {
+    fn drop(&mut self) {
+        self.nodes.drain().for_each(|(_, mut node)| {
+            Self::disconnect_node(&mut node);
+        })
+    }
+}
