@@ -4,6 +4,7 @@ mod jobs_test;
 
 use crate::models::common::*;
 use crate::models::problem::{Costs, Fleet, TransportCost};
+use crate::utils::compare_floats;
 use hashbrown::HashMap;
 use std::cell::UnsafeCell;
 use std::cmp::Ordering::Less;
@@ -53,6 +54,14 @@ impl Job {
             Job::Multi(multi) => &multi.dimens,
         }
     }
+
+    /// Get all places from the job.
+    pub fn places(&self) -> Box<dyn Iterator<Item = &Place> + '_> {
+        match &self {
+            Job::Single(single) => Box::new(single.places.iter()),
+            Job::Multi(multi) => Box::new(multi.jobs.iter().flat_map(|single| single.places.iter())),
+        }
+    }
 }
 
 /// Represents a job place details where and/or when work has to be performed.
@@ -197,11 +206,6 @@ impl Jobs {
         self.jobs.iter().cloned()
     }
 
-    /// Returns all jobs in original order as slice.
-    pub fn all_as_slice(&self) -> &[Job] {
-        self.jobs.as_slice()
-    }
-
     /// Returns range of jobs "near" to given one. Near is defined by costs with relation
     /// transport profile and departure time.
     pub fn neighbors(&self, profile: &Profile, job: &Job, _: Timestamp) -> impl Iterator<Item = &(Job, Cost)> {
@@ -252,10 +256,18 @@ impl Hash for Job {
     }
 }
 
+/// Returns job locations.
+pub fn get_job_locations<'a>(job: &'a Job) -> Box<dyn Iterator<Item = Option<Location>> + 'a> {
+    match job {
+        Job::Single(single) => Box::new(single.places.iter().map(|p| p.location)),
+        Job::Multi(multi) => Box::new(multi.jobs.iter().flat_map(|j| j.places.iter().map(|p| p.location))),
+    }
+}
+
 // TODO: we don't know actual departure and zero-cost when we create job index.
 const DEFAULT_DEPARTURE: Timestamp = 0.;
 const DEFAULT_COST: Cost = 0.;
-const UNREACHABLE_COST: Cost = std::f32::MAX as f64;
+const UNREACHABLE_COST: Cost = f64::MAX;
 
 /// Creates job index.
 fn create_index(
@@ -349,23 +361,37 @@ fn get_cost_between_jobs(
     let outer: Vec<Option<Location>> = get_job_locations(lhs).collect();
     let inner: Vec<Option<Location>> = get_job_locations(rhs).collect();
 
-    outer
+    let (location_cost, duration) = outer
         .iter()
         .flat_map(|o| inner.iter().map(move |i| (*o, *i)))
         .map(|pair| match pair {
-            (Some(from), Some(to)) => get_cost_between_locations(profile, costs, transport, from, to),
-            _ => DEFAULT_COST,
+            (Some(from), Some(to)) => {
+                let total = get_cost_between_locations(profile, costs, transport, from, to);
+                let duration = transport.duration(profile, from, to, DEFAULT_DEPARTURE);
+                (total, duration)
+            }
+            _ => (DEFAULT_COST, 0.),
         })
-        .min_by(|a, b| a.partial_cmp(b).unwrap_or(Less))
-        .unwrap_or(DEFAULT_COST)
-}
+        .min_by(|(a, _), (b, _)| compare_floats(*a, *b))
+        .unwrap_or((DEFAULT_COST, 0.));
 
-/// Returns job locations.
-fn get_job_locations<'a>(job: &'a Job) -> Box<dyn Iterator<Item = Option<Location>> + 'a> {
-    match job {
-        Job::Single(single) => Box::new(single.places.iter().map(|p| p.location)),
-        Job::Multi(multi) => Box::new(multi.jobs.iter().flat_map(|j| j.places.iter().map(|p| p.location))),
-    }
+    let time_cost = lhs
+        .places()
+        .flat_map(|place| place.times.iter())
+        .flat_map(|left| {
+            rhs.places().flat_map(|place| place.times.iter()).map(move |right| match (left, right) {
+                (TimeSpan::Window(left), TimeSpan::Window(right)) => {
+                    // NOTE exclude traveling duration between jobs
+                    (left.distance(right) - duration).max(0.)
+                }
+                _ => 0.,
+            })
+        })
+        .min_by(|a, b| compare_floats(*a, *b))
+        .unwrap_or(0.)
+        * costs.per_waiting_time;
+
+    location_cost + time_cost
 }
 
 fn get_avg_profile_costs(fleet: &Fleet) -> HashMap<usize, Costs> {
